package cz.muni.fi.pb162.radio;

import cz.muni.fi.pb162.radio.parts.*;
/**
 * FM radio with switch, speaker, one line 20 characters long display
 * 
 * @author ubo Pecho, tomorrow@mail.muni.cz .
 * @version 8.10.2009
 */
public class RadioImpl implements Radio
{

    /**
     * FM frequency, on which the radio should receive signal
     */
    private double frequency = 0;
    
    /**
     * Switch, with which the radio can be turn on and off
     */
    private Switch radioSwitch;
    
    /**
     * Speaker of the radio
     */
    private Speaker speaker;
    
    /**
     * Display of the radio
     */
    private Display display;
    
    public RadioImpl(){
        radioSwitch = new Switch();
        speaker = new Speaker(30);
        display = new Display();
    }
    
    /**
     * Constructor that creates new radio with its all parts
     * @param maxVolume the desired maximal volume of the radio
     */
    public RadioImpl(int maxVolume){
        radioSwitch = new Switch();
        speaker = new Speaker(maxVolume);
        display = new Display();
    }
    
    /**
     * Turns the radio on
     */
    public void turnOn(){
        radioSwitch.setTurnedOn(true);
    }
    
    /**
     * Turns the radio off
     */
    public void turnOff(){
        radioSwitch.setTurnedOn(false);
        display.setContent("");
    }
    
    /**
     * Increases the volume of the radio by one point
     */
    public void increaseVolume(){
        if(radioSwitch.isTurnedOn()){
            speaker.setVolume(speaker.getVolume() + 1);
        }
    }
    
    /**
     * Decreases the volume of the radio by one point
     */
    public void decreaseVolume(){
        if(radioSwitch.isTurnedOn()){
            speaker.setVolume(speaker.getVolume() - 1);
        }
    }
    
    /**
     * Sets the frequency, on which the radio should receive signal. 
     * Negative frequencies are ignored
     * @param frequency
     */
    public void setFrequency(double frequency){
        if(radioSwitch.isTurnedOn() && frequency >= 0){
            this.frequency = frequency;
            printFrequencyToDisplay();
        }
    }   
    
    /**
     * Increases the frequency by the given value
     * @param frequency the value, by which should be the frequency increased
     */
    public void increaseFrequency(double frequency){
        if(radioSwitch.isTurnedOn() && frequency >= 0) {
            this.frequency += frequency;
            printFrequencyToDisplay();
        }    
    }
    
    /**
     * Increases the frequency by the given value
     * @param frequency the value, by which should be the frequency increased
     */
    public void increaseFrequency(int frequency){
        if(radioSwitch.isTurnedOn() && frequency >= 0) {
            this.frequency += frequency;
            printFrequencyToDisplay();
        }    
    }
    
    /**
     * Decreases the frequency by the given value
     * @param frequency the value, by which should be the frequency decreased
     */
    public void decreaseFrequency(double frequency){
        if(radioSwitch.isTurnedOn() 
            && frequency >=0 
            && frequency <= this.frequency) {
            this.frequency -= frequency;
            printFrequencyToDisplay();
       }
    }    

    /**
     * Decreases the frequency by the given value
     * @param frequency the value, by which should be the frequency decreased
     */
    public void decreaseFrequency(int frequency){
        if(radioSwitch.isTurnedOn() 
            && frequency >=0 
            && frequency <= this.frequency) {
            this.frequency -= frequency;
            printFrequencyToDisplay();
       }
    }    

    /**
     * prints the frequency to the display
     */
    private void printFrequencyToDisplay(){
        display.setContent(((Double)frequency).toString());
    }    
}
